package com.ruoyi.project.invoice.service;

import com.ruoyi.project.invoice.domain.TApproveReserveInvoice;

import java.util.List;

/**
 * 预约开票申请Service接口
 *
 * @author ruoyi
 * @date 2022-08-29
 */
public interface ITApproveReserveInvoiceService
{
    /**
     * 查询预约开票申请
     *
     * @param id 预约开票申请ID
     * @return 预约开票申请
     */
    public TApproveReserveInvoice selectTApproveReserveInvoiceById(Long id);

    /**
     * 查询预约开票申请列表
     *
     * @param tApproveReserveInvoice 预约开票申请
     * @return 预约开票申请集合
     */
    public List<TApproveReserveInvoice> selectTApproveReserveInvoiceList(TApproveReserveInvoice tApproveReserveInvoice);

    /**
     * 新增预约开票申请
     *
     * @param tApproveReserveInvoice 预约开票申请
     * @return 结果
     */
    public int insertTApproveReserveInvoice(TApproveReserveInvoice tApproveReserveInvoice);

    /**
     * 修改预约开票申请
     *
     * @param tApproveReserveInvoice 预约开票申请
     * @return 结果
     */
    public int updateTApproveReserveInvoice(TApproveReserveInvoice tApproveReserveInvoice);

    /**
     * 批量删除预约开票申请
     *
     * @param ids 需要删除的预约开票申请ID
     * @return 结果
     */
    public int deleteTApproveReserveInvoiceByIds(Long[] ids);

    /**
     * 删除预约开票申请信息
     *
     * @param id 预约开票申请ID
     * @return 结果
     */
    public int deleteTApproveReserveInvoiceById(Long id);

    /**
     * 查询预约开票申请
     *
     * @param invoice 预约开票申请ID
     * @return 预约开票申请
     */
    TApproveReserveInvoice selectTApproveReserveInvoiceByinvoiceId(String invoiceId);
}
